"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const resolve_1 = __importDefault(require("resolve"));
exports.default = () => {
    return {
        name: 'vite:react-jsx',
        enforce: 'pre',
        config: () => ({
            resolve: {
                dedupe: ['react', 'react-dom'],
            },
        }),
        configResolved(config) {
            if (config.command === 'build') {
                Object.assign(this, getBuildPlugin(config));
            }
            else {
                const jsxRE = /\.[tj]sx$/;
                const reactRE = /(^|\n)import React /;
                // Just use React.createElement in serve mode
                this.transform = function (code, id) {
                    if (jsxRE.test(id) && !reactRE.test(code)) {
                        return `import React from 'react'; ` + code;
                    }
                };
            }
        },
    };
};
function getBuildPlugin(config) {
    const babelImport = Promise.resolve().then(() => __importStar(require('@babel/core')));
    const babelTransformJsx = Promise.resolve().then(() => __importStar(require('@babel/plugin-transform-react-jsx')));
    const runtimeId = 'react/jsx-runtime';
    const runtimePath = resolve_1.default.sync('react/jsx-runtime', {
        basedir: config.root,
    });
    return {
        resolveId: id => (id === runtimeId ? id : null),
        load(id) {
            if (id === runtimeId) {
                const exports = ['jsx', 'jsxs', 'Fragment'];
                return [
                    `import * as jsxRuntime from '${runtimePath}'`,
                    // We can't use `export * from` or else any callsite that uses
                    // this module will be compiled to `jsxRuntime.exports.jsx`
                    // instead of the more concise `jsx` alias.
                    ...exports.map(name => `export const ${name} = jsxRuntime.${name}`),
                ].join('\n');
            }
        },
        async transform(code, id) {
            if (/.+\/node_modules\/.+\.jsx?$/.test(id)) {
                let match = code.match(/\b(var|let|const) +(\w+) *= *require\(["']react["']\)/);
                if (match) {
                    return nodeModulesTransform(code, match[2], true);
                }
                match = code.match(/^import (\w+).+? from ["']react["']/m);
                if (match) {
                    return nodeModulesTransform(code, match[1]);
                }
            }
            else if (/\.[tj]sx$/.test(id)) {
                const babel = await babelImport;
                const res = await babel.transformAsync(code, {
                    plugins: [[await babelTransformJsx, { runtime: 'automatic' }]],
                    sourceMaps: config.build.sourcemap,
                });
                if (res === null || res === void 0 ? void 0 : res.code) {
                    return {
                        code: res.code,
                        map: res.map,
                    };
                }
            }
        },
    };
    async function nodeModulesTransform(code, reactAlias, isCommonJS) {
        const reactJsxRE = new RegExp('\\b' + reactAlias + '\\.(createElement|Fragment)\\b', 'g');
        let hasCompiledJsx = false;
        code = code.replace(reactJsxRE, (_, prop) => {
            hasCompiledJsx = true;
            // Replace with "React" so JSX can be reverse compiled.
            return 'React.' + prop;
        });
        // Assume node_modules never contains uncompiled JSX.
        if (!hasCompiledJsx) {
            return;
        }
        // Support modules that use `import {Fragment} from 'react'`
        code = code.replace(/createElement\(Fragment,/g, 'createElement(React.Fragment,');
        const babel = await babelImport;
        // Restore JSX from React.createElement calls
        let res = await babel.transformAsync(code, {
            ast: true,
            code: false,
            plugins: await Promise.all([
                Promise.resolve().then(() => __importStar(require('@babel/plugin-syntax-jsx'))).then(mod => mod.default),
                Promise.resolve().then(() => __importStar(require('./restore-jsx'))),
            ]),
        });
        // Then apply the JSX automatic runtime transform
        if (res === null || res === void 0 ? void 0 : res.ast) {
            const plugins = [
                [await babelTransformJsx, { runtime: 'automatic' }],
            ];
            if (isCommonJS) {
                // Replace this:
                //    import { jsx as _jsx } from "react/jsx-runtime"
                // with this:
                //    var _jsx = require("react/jsx-runtime").jsx
                plugins.push(babelImportToRequire);
            }
            res = await babel.transformFromAstAsync(res.ast, undefined, {
                plugins,
                sourceMaps: config.build.sourcemap,
            });
            if (res === null || res === void 0 ? void 0 : res.code) {
                return {
                    code: res.code,
                    map: res.map,
                };
            }
        }
    }
    function babelImportToRequire({ types: t }) {
        return {
            visitor: {
                ImportDeclaration(path) {
                    const decl = path.node;
                    const spec = decl.specifiers[0];
                    path.replaceWith(t.variableDeclaration('var', [
                        t.variableDeclarator(spec.local, t.memberExpression(t.callExpression(t.identifier('require'), [decl.source]), spec.imported)),
                    ]));
                },
            },
        };
    }
}
//# sourceMappingURL=plugin.js.map